import { Subject } from "rxjs";
import * as io from 'socket.io-client';

/* eslint-disable */
export class TestSocketIoClient {
  socket = null
  responses

  disconnect() {
    this.socket.close();
    this.socket.disconnect();
  }

  connect(url) {
    this.responses = new Subject()

    const ws = io.connect(url)

    ws.on('connecting',     (x) => { console.info('connecting', x);     });
    ws.on('reconnecting',   (x) => { console.info('reconnecting', x);   });
    ws.on('connect_failed', (x) => { console.info('connect_failed',x);  });
    ws.on('error',          (x) => { console.info('error', x);          });
    ws.on('disconnect',     (x) => { console.info('disconnect',x);      });
    ws.on('connect',        (x) => { console.info('connect', x)         });
    
    ws.on('response', (data) => { this.responses.next(data)  });

    this.socket = ws;
  }

  getStreamOfResponsesFromRequest(request) {
    const subject = new Subject();
    
    const id = this.sendRequest(request);
    const subscription = this.responses.subscribe({
      next: (data) => {
        if ( data?.id != id ) { return }
        if ( data?.complete ) return resultsComplete();
        
        const response = JSON.parse( data?.message ? data.message : {} )
        subject.next(response)
      },
      complete: () => resultsComplete(),
      error: () => resultsComplete(),
    })

    function resultsComplete(){
      subject.complete();
      try { subscription.unsubscribe() } 
      catch (error) { console.info('error', error) }
    }

    return subject;
  }

  getAllResponsesFromRequest(request) {
    const id = this.sendRequest(request);
    const results = []

    return new Promise( (resolve) => {
      const subscription = this.responses.subscribe({
        next: (data) => {
          console.info('got next', data)
          if ( data?.id != id ) { return }
          if ( data.complete ) return resultsComplete()

          const response = JSON.parse( data?.message ? data.message : {} )
          results.push(response);
        },
        complete: () => resultsComplete(),
        error: () => resultsComplete(),
      })

      function resultsComplete(){
        console.info('got result', results)
        resolve(results)
        try { subscription.unsubscribe() }
        catch (error) { console.info('error', error) }
      }
    });

  }

  sendRequest(request) {
    const id = Math.round(Math.random() * 100000);
    setTimeout(()=> {
      this.socket.emit('request', {
        id,
        message: JSON.stringify(request)
      })
    }, 0);
    return id;
  }
}